import logging
from optparse import make_option
from django.core.management.base import BaseCommand, CommandError
from django.db import connections, transaction, DEFAULT_DB_ALIAS, DatabaseError
from vdw.assessments.models import Assessment
from vdw.samples.models import Project
from vdw.signals import pre_delete_project

log = logging.getLogger(__name__)


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--database', action='store', dest='database',
                    default=DEFAULT_DB_ALIAS,
                    help='Specifies the target database to delete the proect '
                         'from.'),
        make_option('--remove-metrics', action='store', dest='remove-metrics',
                    default=False, help='Forces the removal of metrics on '
                    'samples, this only applies to production')
    )

    def handle(self, *args, **options):
        if not args:
            raise CommandError('A project ID must be specified')

        project_id = args[0]

        # If any of the samples within the project contain knowledge capture
        # data then refuse to delete the project and notify the user why.
        if Assessment.objects.filter(
                sample_result__sample__project_id=project_id).exists():
            print("This project cannot be deleted because it has knowledge "
                  "capture data associated with samples it contains. If you "
                  "are absolutely sure you want to delete this project, "
                  "manually delete the knowledge capture data on this "
                  "project's samples and then rerun this command.")
            return

        database = options.get('database')
        remove_metrics = options.get('remove-metrics')
        cursor = connections[database].cursor()

        with transaction.commit_manually(database):
            try:
                # Remove all variants in cohorts associated with the project
                cursor.execute('''
                    DELETE FROM cohort_variant WHERE cohort_id IN (
                        SELECT cohort.id FROM cohort INNER JOIN project ON
                            (cohort.project_id = project.id)
                            WHERE project.id = %s
                    )
                ''', [project_id])

                # Remove samples in cohorts associated with the project
                cursor.execute('''
                    DELETE FROM cohort_sample WHERE sample_id IN (
                        SELECT sample.id FROM sample INNER JOIN project ON
                            (sample.project_id = project.id)
                            WHERE project.id = %s
                    )
                ''', [project_id])

                # Remove all cohorts associated with the project
                cursor.execute('''
                    DELETE FROM cohort WHERE project_id = %s
                ''', [project_id])

                # Remove metrics on samples, this only applies to production
                if remove_metrics:
                    cursor.execute('''
                        DELETE FROM metrics_sample_load WHERE sample_id IN (
                            SELECT sample.id FROM sample INNER JOIN project ON
                                (sample.project_id = project.id)
                                WHERE project.id = %s
                        )
                    ''', [project_id])

                # Remove result scores of samples
                cursor.execute('''
                    DELETE FROM result_score WHERE result_id IN (
                        SELECT id from sample_result WHERE sample_id IN (
                            SELECT id FROM sample WHERE project_id = %s
                        )
                    )
                ''', [project_id])

                # Remove results of samples
                cursor.execute('''
                    DELETE FROM sample_result WHERE sample_id IN (
                        SELECT sample.id FROM sample INNER JOIN project ON
                            (sample.project_id = project.id)
                            WHERE project.id = %s
                    )
                ''', [project_id])

                # Remove sample manifest
                cursor.execute('''
                    DELETE FROM sample_manifest WHERE sample_id IN (
                        SELECT sample.id FROM sample INNER JOIN project ON
                            (sample.project_id = project.id)
                            WHERE project.id = %s
                    )
                ''', [project_id])

                # Alert listeners to project deletion.
                pre_delete_project.send(
                    sender=Project, pk=project_id, cursor=cursor)

                # Remove samples
                cursor.execute('''
                    DELETE FROM sample WHERE project_id = %s
                ''', [project_id])

                # Remove batches
                cursor.execute('''
                    DELETE FROM batch WHERE project_id = %s
                ''', [project_id])

                # Remove the project
                cursor.execute('''
                    DELETE FROM project WHERE id = %s
                ''', [project_id])

                transaction.commit()
            except DatabaseError:
                transaction.rollback()
                log.exception("Error occurred while deleting project")
                raise



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    depends_on = (
        ('genome', '0001_initial'),
        ('literature', '0001_initial'),
        ('phenotypes', '0001_initial'),
        ('genes', '0001_initial'),
    )

    def forwards(self, orm):

        # Adding model 'EffectImpact'
        db.create_table('effect_impact', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=100)),
        ))
        db.send_create_signal('variants', ['EffectImpact'])

        # Adding model 'EffectRegion'
        db.create_table('effect_region', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=100)),
        ))
        db.send_create_signal('variants', ['EffectRegion'])

        # Adding model 'Effect'
        db.create_table('effect', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('impact', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['variants.EffectImpact'], null=True, blank=True)),
            ('region', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['variants.EffectRegion'], null=True, blank=True)),
        ))
        db.send_create_signal('variants', ['Effect'])

        # Adding model 'VariantType'
        db.create_table('variant_type', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=20)),
        ))
        db.send_create_signal('variants', ['VariantType'])

        # Adding model 'Variant'
        db.create_table('variant', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('chr', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['genome.Chromosome'])),
            ('pos', self.gf('django.db.models.fields.IntegerField')()),
            ('ref', self.gf('django.db.models.fields.TextField')(db_index=True)),
            ('alt', self.gf('django.db.models.fields.TextField')(db_index=True)),
            ('md5', self.gf('django.db.models.fields.CharField')(max_length=32)),
            ('rsid', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['variants.VariantType'], null=True)),
            ('liftover', self.gf('django.db.models.fields.NullBooleanField')(null=True, blank=True)),
        ))
        db.send_create_signal('variants', ['Variant'])

        # Adding unique constraint on 'Variant', fields ['chr', 'pos', 'ref', 'alt']
        db.create_unique('variant', ['chr_id', 'pos', 'ref', 'alt'])

        # Adding M2M table for field articles on 'Variant'
        db.create_table('variant_pubmed', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('variant', models.ForeignKey(orm['variants.variant'], null=False)),
            ('pubmed', models.ForeignKey(orm['literature.pubmed'], null=False))
        ))
        db.create_unique('variant_pubmed', ['variant_id', 'pubmed_id'])

        # Adding model 'VariantPhenotype'
        db.create_table('variant_phenotype', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('phenotype', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['phenotypes.Phenotype'])),
            ('hgmd_id', self.gf('django.db.models.fields.CharField')(max_length=30, null=True, blank=True)),
            ('variant', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['variants.Variant'])),
        ))
        db.send_create_signal('variants', ['VariantPhenotype'])

        # Adding model 'ThousandG'
        db.create_table('1000g', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('variant', self.gf('django.db.models.fields.related.ForeignKey')(related_name='thousandg', to=orm['variants.Variant'])),
            ('an', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('ac', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('af', self.gf('django.db.models.fields.FloatField')(null=True, db_index=True)),
            ('aa', self.gf('django.db.models.fields.TextField')(null=True)),
            ('amr_af', self.gf('django.db.models.fields.FloatField')(null=True, db_index=True)),
            ('asn_af', self.gf('django.db.models.fields.FloatField')(null=True, db_index=True)),
            ('afr_af', self.gf('django.db.models.fields.FloatField')(null=True, db_index=True)),
            ('eur_af', self.gf('django.db.models.fields.FloatField')(null=True, db_index=True)),
        ))
        db.send_create_signal('variants', ['ThousandG'])

        # Adding model 'EVS'
        db.create_table('evs', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('variant', self.gf('django.db.models.fields.related.ForeignKey')(related_name='evs', to=orm['variants.Variant'])),
            ('ea_ac_ref', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('ea_ac_alt', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('aa_ac_ref', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('aa_ac_alt', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('all_ac_ref', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('all_ac_alt', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('ea_maf', self.gf('django.db.models.fields.FloatField')(null=True, db_index=True)),
            ('aa_maf', self.gf('django.db.models.fields.FloatField')(null=True, db_index=True)),
            ('all_maf', self.gf('django.db.models.fields.FloatField')(null=True, db_index=True)),
            ('clinical_info', self.gf('django.db.models.fields.TextField')(null=True)),
        ))
        db.send_create_signal('variants', ['EVS'])

        # Adding model 'PolyPhen2'
        db.create_table('polyphen2', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('variant', self.gf('django.db.models.fields.related.ForeignKey')(related_name='polyphen2', to=orm['variants.Variant'])),
            ('score', self.gf('django.db.models.fields.FloatField')(null=True, db_index=True)),
            ('refaa', self.gf('django.db.models.fields.CharField')(max_length=2, null=True)),
        ))
        db.send_create_signal('variants', ['PolyPhen2'])

        # Adding model 'Sift'
        db.create_table('sift', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('variant', self.gf('django.db.models.fields.related.ForeignKey')(related_name='sift', to=orm['variants.Variant'])),
            ('score', self.gf('django.db.models.fields.FloatField')(null=True, db_index=True)),
            ('refaa', self.gf('django.db.models.fields.CharField')(max_length=2, null=True)),
            ('varaa', self.gf('django.db.models.fields.CharField')(max_length=2, null=True)),
        ))
        db.send_create_signal('variants', ['Sift'])

        # Adding model 'FunctionalClass'
        db.create_table('variant_functional_class', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('code', self.gf('django.db.models.fields.IntegerField')(null=True)),
            ('order', self.gf('django.db.models.fields.IntegerField')(default=0)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=100)),
        ))
        db.send_create_signal('variants', ['FunctionalClass'])

        # Adding model 'VariantEffect'
        db.create_table('variant_effect', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('variant', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='effects', null=True, to=orm['variants.Variant'])),
            ('codon_change', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('amino_acid_change', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('exon', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['genes.Exon'], null=True, blank=True)),
            ('transcript', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['genes.Transcript'], null=True, blank=True)),
            ('gene', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['genes.Gene'], null=True, blank=True)),
            ('effect', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['variants.Effect'], null=True, blank=True)),
            ('functional_class', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['variants.FunctionalClass'], null=True, blank=True)),
        ))
        db.send_create_signal('variants', ['VariantEffect'])


    def backwards(self, orm):

        # Removing unique constraint on 'Variant', fields ['chr', 'pos', 'ref', 'alt']
        db.delete_unique('variant', ['chr_id', 'pos', 'ref', 'alt'])

        # Deleting model 'EffectImpact'
        db.delete_table('effect_impact')

        # Deleting model 'EffectRegion'
        db.delete_table('effect_region')

        # Deleting model 'Effect'
        db.delete_table('effect')

        # Deleting model 'VariantType'
        db.delete_table('variant_type')

        # Deleting model 'Variant'
        db.delete_table('variant')

        # Removing M2M table for field articles on 'Variant'
        db.delete_table('variant_pubmed')

        # Deleting model 'VariantPhenotype'
        db.delete_table('variant_phenotype')

        # Deleting model 'ThousandG'
        db.delete_table('1000g')

        # Deleting model 'EVS'
        db.delete_table('evs')

        # Deleting model 'PolyPhen2'
        db.delete_table('polyphen2')

        # Deleting model 'Sift'
        db.delete_table('sift')

        # Deleting model 'FunctionalClass'
        db.delete_table('variant_functional_class')

        # Deleting model 'VariantEffect'
        db.delete_table('variant_effect')


    models = {
        'genes.exon': {
            'Meta': {'object_name': 'Exon', 'db_table': "'exon'"},
            'end': ('django.db.models.fields.IntegerField', [], {}),
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'index': ('django.db.models.fields.IntegerField', [], {}),
            'start': ('django.db.models.fields.IntegerField', [], {})
        },
        'genes.gene': {
            'Meta': {'object_name': 'Gene', 'db_table': "'gene'"},
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'db_table': "'gene_pubmed'", 'symmetrical': 'False'}),
            'chr': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genome.Chromosome']"}),
            'families': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.GeneFamily']", 'symmetrical': 'False', 'blank': 'True'}),
            'hgnc_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'phenotypes': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['phenotypes.Phenotype']", 'through': "orm['genes.GenePhenotype']", 'symmetrical': 'False'}),
            'symbol': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            'synonyms': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.Synonym']", 'db_table': "'gene_synonym'", 'symmetrical': 'False'})
        },
        'genes.genefamily': {
            'Meta': {'object_name': 'GeneFamily', 'db_table': "'gene_family'"},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'tag': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True'})
        },
        'genes.genephenotype': {
            'Meta': {'object_name': 'GenePhenotype', 'db_table': "'gene_phenotype'"},
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']"}),
            'hgmd_id': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'phenotype': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['phenotypes.Phenotype']"})
        },
        'genes.synonym': {
            'Meta': {'object_name': 'Synonym', 'db_table': "'synonym'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'})
        },
        'genes.transcript': {
            'Meta': {'object_name': 'Transcript', 'db_table': "'transcript'"},
            'coding_end': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'coding_end_status': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'coding_start': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'coding_start_status': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'end': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'exon_count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'exons': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.Exon']", 'db_table': "'transcript_exon'", 'symmetrical': 'False'}),
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refseq_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'start': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'strand': ('django.db.models.fields.CharField', [], {'max_length': '1', 'null': 'True', 'blank': 'True'})
        },
        'genome.chromosome': {
            'Meta': {'ordering': "['order']", 'object_name': 'Chromosome', 'db_table': "'chromosome'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '2', 'db_index': 'True'})
        },
        'literature.pubmed': {
            'Meta': {'object_name': 'PubMed', 'db_table': "'pubmed'"},
            'pmid': ('django.db.models.fields.IntegerField', [], {'primary_key': 'True'})
        },
        'phenotypes.phenotype': {
            'Meta': {'object_name': 'Phenotype', 'db_table': "'phenotype'"},
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'symmetrical': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'hpo_id': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'term': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '1000'})
        },
        'variants.effect': {
            'Meta': {'ordering': "['order']", 'object_name': 'Effect', 'db_table': "'effect'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'impact': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.EffectImpact']", 'null': 'True', 'blank': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'region': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.EffectRegion']", 'null': 'True', 'blank': 'True'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.effectimpact': {
            'Meta': {'ordering': "['order']", 'object_name': 'EffectImpact', 'db_table': "'effect_impact'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.effectregion': {
            'Meta': {'ordering': "['order']", 'object_name': 'EffectRegion', 'db_table': "'effect_region'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.evs': {
            'Meta': {'object_name': 'EVS', 'db_table': "'evs'"},
            'aa_ac_alt': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'aa_ac_ref': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'aa_maf': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'all_ac_alt': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'all_ac_ref': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'all_maf': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'clinical_info': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'ea_ac_alt': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'ea_ac_ref': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'ea_maf': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'evs'", 'to': "orm['variants.Variant']"})
        },
        'variants.functionalclass': {
            'Meta': {'ordering': "['order']", 'object_name': 'FunctionalClass', 'db_table': "'variant_functional_class'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.polyphen2': {
            'Meta': {'object_name': 'PolyPhen2', 'db_table': "'polyphen2'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refaa': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'score': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'polyphen2'", 'to': "orm['variants.Variant']"})
        },
        'variants.sift': {
            'Meta': {'object_name': 'Sift', 'db_table': "'sift'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refaa': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'score': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'varaa': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'sift'", 'to': "orm['variants.Variant']"})
        },
        'variants.thousandg': {
            'Meta': {'object_name': 'ThousandG', 'db_table': "'1000g'"},
            'aa': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'ac': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'afr_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'amr_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'an': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'asn_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'eur_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'thousandg'", 'to': "orm['variants.Variant']"})
        },
        'variants.variant': {
            'Meta': {'unique_together': "(('chr', 'pos', 'ref', 'alt'),)", 'object_name': 'Variant', 'db_table': "'variant'"},
            'alt': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'db_table': "'variant_pubmed'", 'symmetrical': 'False'}),
            'chr': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genome.Chromosome']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'liftover': ('django.db.models.fields.NullBooleanField', [], {'null': 'True', 'blank': 'True'}),
            'md5': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'phenotypes': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['phenotypes.Phenotype']", 'through': "orm['variants.VariantPhenotype']", 'symmetrical': 'False'}),
            'pos': ('django.db.models.fields.IntegerField', [], {}),
            'ref': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'rsid': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.VariantType']", 'null': 'True'})
        },
        'variants.varianteffect': {
            'Meta': {'object_name': 'VariantEffect', 'db_table': "'variant_effect'"},
            'amino_acid_change': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'codon_change': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'effect': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.Effect']", 'null': 'True', 'blank': 'True'}),
            'exon': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Exon']", 'null': 'True', 'blank': 'True'}),
            'functional_class': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.FunctionalClass']", 'null': 'True', 'blank': 'True'}),
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'transcript': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Transcript']", 'null': 'True', 'blank': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'effects'", 'null': 'True', 'to': "orm['variants.Variant']"})
        },
        'variants.variantphenotype': {
            'Meta': {'object_name': 'VariantPhenotype', 'db_table': "'variant_phenotype'"},
            'hgmd_id': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'phenotype': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['phenotypes.Phenotype']"}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.Variant']"})
        },
        'variants.varianttype': {
            'Meta': {'ordering': "['order']", 'object_name': 'VariantType', 'db_table': "'variant_type'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        }
    }

    complete_apps = ['variants']



from vdw.pipeline import Channel

VARIANT_CHANNEL = Channel('variants', ['manifest_path', 'database'])


# Setup local handlers to watch the channels of interest..
from vdw.samples.pipeline import SAMPLE_CHANNEL
from . import handlers
handlers.load_variants.watch(SAMPLE_CHANNEL)
handlers.load_effects.watch(VARIANT_CHANNEL)



import re
import logging
from optparse import make_option
from django.db import transaction, connections, DEFAULT_DB_ALIAS
from django.core.management.base import BaseCommand
from vdw.genes.models import Gene, GenePhenotype
from vdw.literature.models import PubMed
from vdw.phenotypes.models import Phenotype
from vdw.variants.models import Variant, VariantPhenotype

BATCH_SIZE = 1000

log = logging.getLogger(__name__)


def load_hgmd_phenotypes(label, keys, cursor, using):
    count = 0
    total = 0

    # Local storage for new instances
    pmids = {}
    phenotypes = {}

    while True:
        rows = cursor.fetchmany(100)
        if not rows:
            break

        for row in rows:
            record = dict(zip(keys, row))

            pubmed = gene = variant = None
            new_phenotype = saved = False

            # PubMed IDs
            if record['pubmed_id']:
                pubmed = PubMed(pk=record['pubmed_id'])
                pubmed._state.db = using
            # Some records have a bogus PMID. Only proces the valid ones.
            elif type(record['pubmed']) is int or record['pubmed'].isdigit():
                pmid = int(record['pubmed'])
                if pmid in pmids:
                    pubmed = PubMed(pk=pmids[pmid])
                    pubmed._state.db = using
                else:
                    pubmed = PubMed(pmid=pmid)
                    pubmed.save()
                    pmids[pmid] = pubmed.pk

            # Phenotypes
            if record['phenotype_id']:
                phenotype = Phenotype(pk=record['phenotype_id'])
                phenotype._state.db = using
            else:
                new_phenotype = True
                term = record['phenotype']
                if term in phenotypes:
                    phenotype = Phenotype(pk=phenotypes[term])
                    phenotype._state.db = using
                else:
                    phenotype = Phenotype(term=term)
                    phenotype.save()
                    phenotypes[term] = phenotype.pk

            # Variants
            variant = Variant(pk=record['variant_id'])
            variant._state.db = using
            if new_phenotype or not VariantPhenotype.objects\
                    .filter(variant=variant, phenotype=phenotype,
                            hgmd_id=record['hgmd_id']).exists():
                vp = VariantPhenotype(variant=variant, phenotype=phenotype,
                                      hgmd_id=record['hgmd_id'])
                vp.save()
                saved = True

            # Genes
            if record['gene_id']:
                gene = Gene(pk=record['gene_id'])
                gene._state.db = using

                if new_phenotype or not GenePhenotype.objects\
                        .filter(gene=gene, phenotype=phenotype,
                                hgmd_id=record['hgmd_id']).exists():
                    gp = GenePhenotype(gene=gene, phenotype=phenotype,
                                       hgmd_id=record['hgmd_id'])
                    gp.save()
                    saved = True

            # Associate articles with other objects
            if pubmed:
                phenotype.articles.add(pubmed)
                if variant:
                    variant.articles.add(pubmed)
                if gene:
                    gene.articles.add(pubmed)

            total += 1
            if saved:
                count += 1

                if count % BATCH_SIZE == 0:
                    transaction.commit()

        log.debug('Loading {0}...{1}/{2}'.format(label, count, total))

    # Print a newline for the terminal prompt
    print


class Command(BaseCommand):
    """Cleans and loads phenotype terms from various sources into a uniform
    set of tables.
    """
    option_list = BaseCommand.option_list + (
        make_option('--database', action='store', dest='database',
                    default=DEFAULT_DB_ALIAS, help='Nominates a database to '
                    'print the SQL for.  Defaults to the "default" database.'),
        make_option('--hpo', action='store_true', default=False,
                    help='Reload HPO phenotypes'),
        make_option('--hgmd-snp', action='store_true', default=False,
                    help='Load HGMD phenotypes and associations for SNPs.'),
        make_option('--hgmd-indel', action='store_true', default=False,
                    help='Load HGMD phenotypes and associations for INDELs.'),
    )

    def load_hpo(self, cursor, using):
        keys = ['gene_id', 'hpo_terms']

        # Fetch, parse and load only genes that cleanly map to the gene table.
        # Attempting to join against the synonym table may lead to ambiguity
        # as to which approved gene is the correct one.
        cursor.execute('''
            SELECT "gene"."id", "hpo_terms"
            FROM "raw"."hpo_gene_phenotypes"
                LEFT OUTER JOIN "gene"
                    ON ("entrez_gene_symbol" = "gene"."symbol")
            WHERE "gene"."symbol" IS NOT NULL
        ''')

        phenotype_counter = 1
        gene_phenotype_counter = 1

        phenotype_fout = open('phenotype.txt', 'w+')
        gene_phenotype_fout = open('gene_phenotype.txt', 'w+')

        phenotype_header_keys = ['id', 'term', 'description', 'hpo_id']
        gene_phenotype_header_keys = ['id', 'gene_id', 'phenotype_id']

        phenotype_fout.write('\t'.join(phenotype_header_keys) + '\n')
        gene_phenotype_fout.write('\t'.join(gene_phenotype_header_keys) + '\n')

        phenotype_ids = {}

        hpo_term_re = re.compile('(.*?)\(HP:(\d+)\)$')

        while True:
            rows = cursor.fetchmany(100)
            if not rows:
                break

            for row in rows:
                source_record = dict(zip(keys, row))

                hpo_terms = [hpo_term_re.match(term).groups()
                             for term in source_record['hpo_terms'].split(';')]
                for term, hpo_id in hpo_terms:
                    # Write term as new
                    if hpo_id not in phenotype_ids:
                        phenotype_ids[hpo_id] = phenotype_counter
                        phenotype_fout.write('\t'.join([str(phenotype_counter),
                                             term, '\\N', hpo_id]) + '\n')
                        phenotype_counter += 1
                    gene_phenotype_fout.write('\t'.join([
                        str(gene_phenotype_counter),
                        str(source_record['gene_id']),
                        str(phenotype_ids[hpo_id])
                    ]) + '\n')
                    gene_phenotype_counter += 1

        phenotype_fout.flush()
        gene_phenotype_fout.flush()

        phenotype_fout.seek(0)
        gene_phenotype_fout.seek(0)

        phenotype_fout.readline()
        gene_phenotype_fout.readline()

        cursor.copy_from(phenotype_fout, 'phenotype',
                         columns=phenotype_header_keys)
        cursor.copy_from(gene_phenotype_fout, 'gene_phenotype')

        return phenotype_counter

    load_hpo.short_name = 'HPO'

    def load_hgmd_snp(self, cursor, using=None):
        cursor.execute('''
            select
                hgmd.acc_num,
                variant.id,
                hgmd.disease,
                phenotype.id,
                gene.id as gene_id,
                pubmed.pmid,
                hgmd.pmid
            from (
                select
                    v.id,
                    substr(ve.hgvs_c, 3) as hgvs,
                    c.value as chr,
                    t.refseq_id
                from variant_effect ve
                    inner join transcript t on (ve.transcript_id = t.id)
                    inner join variant v on (ve.variant_id = v.id)
                    inner join chromosome c on (v.chr_id = c.id)
                    inner join variant_type vt on (v.type_id = vt.id)
                where vt.value = 'SNP'
                    and ve.hgvs_c is not null
            ) variant inner join
            (
                select
                    mut.acc_num,
                    trim(both from regexp_replace(mut.disease, '\s*\?$', '')) as disease, # noqa
                    _mut.hgvs,
                    mut.gene,
                    mut.pmid,
                    _mut."refCORE" || '.' || _mut."refVER"::text as refseq_id
                from raw.hgmd_mutation mut
                    inner join raw.hgmd_mutnomen _mut
                        on (mut.acc_num = _mut.acc_num)
                    inner join raw.hgmd_hg19_coords hg19
                        on (mut.acc_num = hg19.acc_num)
                where _mut.hgvs is not null
            ) hgmd on (hgmd.refseq_id = variant.refseq_id
                       and hgmd.hgvs = variant.hgvs)
            left outer join gene on (hgmd.gene = gene.symbol)
            left outer join pubmed on (hgmd.pmid = pubmed.pmid::text)
            left outer join phenotype on (hgmd.disease = phenotype.term)
        ''')

        keys = ['hgmd_id', 'variant_id', 'phenotype', 'phenotype_id',
                'gene_id', 'pubmed_id', 'pubmed']

        load_hgmd_phenotypes('HGMD SNP', keys, cursor, using)

    load_hgmd_snp.short_name = 'HGMD SNP'

    def load_hgmd_indel(self, cursor, using=None):
        # The local variant has some overlap with with the HGMD indel
        # the greater of the two lengths for alt and ref must be used
        # Note, Postgres 9.2 has introduces int8range and && operator which
        # simplifies this logic.
        cursor.execute('''
            select
                HGMD.hgmd_id,
                V.id,
                HGMD.disease,
                phenotype.id,
                gene.id,
                pubmed.pmid,
                HGMD.pmid
            from (
                select
                    v.id,
                    c.value as chr,
                    pos as start,
                    pos + greatest(length(ref), length(alt)) as end
                from variant v
                    inner join chromosome c on (v.chr_id = c.id)
                    inner join variant_type vt on (v.type_id = vt.id)
                where vt.value = 'INDEL'
            ) V inner join
            (
                select
                    m.acc_num as hgmd_id,
                    trim(both from regexp_replace(m.disease, '\s*\?$', ''))
                        as disease,
                    m.gene,
                    m.pmid,
                    c.chromosome as chr,
                    c."coordSTART" as start,
                    c."coordEND" as end
                from raw.hgmd_indel m
                    inner join raw.hgmd_hg19_coords c
                        on (m.acc_num = c.acc_num)
                where m.disease not like '%%?'
            ) HGMD on (V.chr = HGMD.chr and (
                -- overlap
                (V.start <= HGMD.end and V.end >= HGMD.end)
                -- overlap
                or (HGMD.start <= V.end and HGMD.end >= V.end)
                -- V contained in HGMD
                or (V.start >= HGMD.start and V.end <= HGMD.end)
                -- HGMD contain in V
                or (HGMD.start >= V.start and HGMD.end <= V.end)
            ))
            left outer join gene on (HGMD.gene = gene.symbol)
            left outer join pubmed on (HGMD.pmid = pubmed.pmid::text)
            left outer join phenotype
                on (lower(HGMD.disease) = lower(phenotype.term))
        ''')

        keys = ['hgmd_id', 'variant_id', 'phenotype', 'phenotype_id',
                'gene_id', 'pubmed_id', 'pubmed']

        load_hgmd_phenotypes('HGMD INDEL', keys, cursor, using)

    load_hgmd_indel.short_name = 'HGMD INDEL'

    def handle(self, *args, **options):
        using = options.get('database')
        load_hpo = self.load_hpo \
            if options.get('hpo') else None
        load_hgmd_snp = self.load_hgmd_snp \
            if options.get('hgmd_snp') else None
        load_hgmd_indel = self.load_hgmd_indel \
            if options.get('hgmd_indel') else None

        connection = connections[using]
        cursor = connection.cursor()

        for handler in (load_hpo, load_hgmd_snp, load_hgmd_indel):
            if not handler:
                continue

            with transaction.commit_manually(using):
                try:
                    handler(cursor, using)
                    transaction.commit()
                except Exception:
                    transaction.rollback()
                    log.exception(
                        'Failed to load {0}'.format(handler.short_name))



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'GeneSet.created'
        db.alter_column('geneset', 'created', self.gf('django.db.models.fields.DateTimeField')())

        # Changing field 'GeneSet.modified'
        db.alter_column('geneset', 'modified', self.gf('django.db.models.fields.DateTimeField')())

    def backwards(self, orm):

        # Changing field 'GeneSet.created'
        db.alter_column('geneset', 'created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True))

        # Changing field 'GeneSet.modified'
        db.alter_column('geneset', 'modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True))

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'avocado.datacontext': {
            'Meta': {'object_name': 'DataContext'},
            'archived': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'composite': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'db_column': "'_count'"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'json': ('jsonfield.fields.JSONField', [], {'default': '{}', 'null': 'True', 'blank': 'True'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'session': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'datacontext+'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'genes.exon': {
            'Meta': {'object_name': 'Exon', 'db_table': "'exon'"},
            'end': ('django.db.models.fields.IntegerField', [], {}),
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'index': ('django.db.models.fields.IntegerField', [], {}),
            'start': ('django.db.models.fields.IntegerField', [], {})
        },
        'genes.gene': {
            'Meta': {'object_name': 'Gene', 'db_table': "'gene'"},
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'db_table': "'gene_pubmed'", 'symmetrical': 'False'}),
            'chr': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genome.Chromosome']"}),
            'families': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.GeneFamily']", 'symmetrical': 'False', 'blank': 'True'}),
            'hgnc_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'phenotypes': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['phenotypes.Phenotype']", 'through': "orm['genes.GenePhenotype']", 'symmetrical': 'False'}),
            'symbol': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            'synonyms': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.Synonym']", 'db_table': "'gene_synonym'", 'symmetrical': 'False'})
        },
        'genes.genefamily': {
            'Meta': {'object_name': 'GeneFamily', 'db_table': "'gene_family'"},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'tag': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True'})
        },
        'genes.genephenotype': {
            'Meta': {'object_name': 'GenePhenotype', 'db_table': "'gene_phenotype'"},
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']"}),
            'hgmd_id': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'phenotype': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['phenotypes.Phenotype']"})
        },
        'genes.geneset': {
            'Meta': {'ordering': "('user', 'name')", 'object_name': 'GeneSet', 'db_table': "'geneset'"},
            'context': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['avocado.DataContext']", 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'count': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'genes': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.Gene']", 'through': "orm['genes.GeneSetObject']", 'symmetrical': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'genes.genesetobject': {
            'Meta': {'object_name': 'GeneSetObject', 'db_table': "'geneset_setobject'"},
            'added': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_set': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.GeneSet']", 'db_column': "'set_id'"}),
            'removed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'set_object': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']", 'db_column': "'object_id'"})
        },
        'genes.synonym': {
            'Meta': {'object_name': 'Synonym', 'db_table': "'synonym'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'})
        },
        'genes.transcript': {
            'Meta': {'object_name': 'Transcript', 'db_table': "'transcript'"},
            'coding_end': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'coding_end_status': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'coding_start': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'coding_start_status': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'end': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'exon_count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'exons': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.Exon']", 'db_table': "'transcript_exon'", 'symmetrical': 'False'}),
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refseq_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'start': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'strand': ('django.db.models.fields.CharField', [], {'max_length': '1', 'null': 'True', 'blank': 'True'})
        },
        'genome.chromosome': {
            'Meta': {'ordering': "['order']", 'object_name': 'Chromosome', 'db_table': "'chromosome'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '2', 'db_index': 'True'})
        },
        'literature.pubmed': {
            'Meta': {'object_name': 'PubMed', 'db_table': "'pubmed'"},
            'pmid': ('django.db.models.fields.IntegerField', [], {'primary_key': 'True'})
        },
        'phenotypes.phenotype': {
            'Meta': {'object_name': 'Phenotype', 'db_table': "'phenotype'"},
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'symmetrical': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'hpo_id': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'term': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '1000'})
        }
    }

    complete_apps = ['genes']


import logging
import sys
from optparse import make_option
from django.core.management import call_command
from django.core.management.base import BaseCommand
from vdw.variants.models import Variant
from vdw.samples.models import Priority

log = logging.getLogger(__name__)


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--force', action='store_true', dest='force_update',
                    default=False,
                    help='Force re-prioritization of existing scores'),
        make_option('--database', action='store', dest='database',
                    default=None, help='Specifies the target database.'),
    )

    def handle(self, *args, **options):
        database = options.get('database')
        if not database:
            log.error('A database must be specified.')
            sys.exit(1)
        force_update = options['force_update']
        count = 0

        if force_update:
            variants = Variant.objects.using(database)
        else:
            variants = Variant.objects.using(database).filter(priority=None)

        for variant in variants.iterator():
            try:
                priority = variant.priority
            except Priority.DoesNotExist:
                priority = Priority(variant=variant)

            priority.score = variant.calculate_priority()
            priority.save(using=database)
            count += 1

        if count:
            log.debug('{0} variants prioritized'.format(count))

        call_command('results stats', database=database)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'VariantEffect.hgvs_c'
        db.add_column('variant_effect', 'hgvs_c',
                      self.gf('django.db.models.fields.CharField')(db_index=True, max_length=200, null=True, blank=True),
                      keep_default=False)

        # Adding field 'VariantEffect.hgvs_p'
        db.add_column('variant_effect', 'hgvs_p',
                      self.gf('django.db.models.fields.CharField')(db_index=True, max_length=200, null=True, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'VariantEffect.hgvs_c'
        db.delete_column('variant_effect', 'hgvs_c')

        # Deleting field 'VariantEffect.hgvs_p'
        db.delete_column('variant_effect', 'hgvs_p')


    models = {
        'genes.exon': {
            'Meta': {'object_name': 'Exon', 'db_table': "'exon'"},
            'end': ('django.db.models.fields.IntegerField', [], {}),
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'index': ('django.db.models.fields.IntegerField', [], {}),
            'start': ('django.db.models.fields.IntegerField', [], {})
        },
        'genes.gene': {
            'Meta': {'object_name': 'Gene', 'db_table': "'gene'"},
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'db_table': "'gene_pubmed'", 'symmetrical': 'False'}),
            'chr': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genome.Chromosome']"}),
            'families': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.GeneFamily']", 'symmetrical': 'False', 'blank': 'True'}),
            'hgnc_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'phenotypes': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['phenotypes.Phenotype']", 'through': "orm['genes.GenePhenotype']", 'symmetrical': 'False'}),
            'symbol': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            'synonyms': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.Synonym']", 'db_table': "'gene_synonym'", 'symmetrical': 'False'})
        },
        'genes.genefamily': {
            'Meta': {'object_name': 'GeneFamily', 'db_table': "'gene_family'"},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'tag': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True'})
        },
        'genes.genephenotype': {
            'Meta': {'object_name': 'GenePhenotype', 'db_table': "'gene_phenotype'"},
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']"}),
            'hgmd_id': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'phenotype': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['phenotypes.Phenotype']"})
        },
        'genes.synonym': {
            'Meta': {'object_name': 'Synonym', 'db_table': "'synonym'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'})
        },
        'genes.transcript': {
            'Meta': {'object_name': 'Transcript', 'db_table': "'transcript'"},
            'coding_end': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'coding_end_status': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'coding_start': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'coding_start_status': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'end': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'exon_count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'exons': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.Exon']", 'db_table': "'transcript_exon'", 'symmetrical': 'False'}),
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refseq_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'start': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'strand': ('django.db.models.fields.CharField', [], {'max_length': '1', 'null': 'True', 'blank': 'True'})
        },
        'genome.chromosome': {
            'Meta': {'ordering': "['order']", 'object_name': 'Chromosome', 'db_table': "'chromosome'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '2', 'db_index': 'True'})
        },
        'literature.pubmed': {
            'Meta': {'object_name': 'PubMed', 'db_table': "'pubmed'"},
            'pmid': ('django.db.models.fields.IntegerField', [], {'primary_key': 'True'})
        },
        'phenotypes.phenotype': {
            'Meta': {'object_name': 'Phenotype', 'db_table': "'phenotype'"},
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'symmetrical': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'hpo_id': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'term': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '1000'})
        },
        'variants.effect': {
            'Meta': {'ordering': "['order']", 'object_name': 'Effect', 'db_table': "'effect'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'impact': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.EffectImpact']", 'null': 'True', 'blank': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'region': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.EffectRegion']", 'null': 'True', 'blank': 'True'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.effectimpact': {
            'Meta': {'ordering': "['order']", 'object_name': 'EffectImpact', 'db_table': "'effect_impact'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.effectregion': {
            'Meta': {'ordering': "['order']", 'object_name': 'EffectRegion', 'db_table': "'effect_region'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.evs': {
            'Meta': {'object_name': 'EVS', 'db_table': "'evs'"},
            'aa_ac_alt': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'aa_ac_ref': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'aa_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'aa_gtc': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'all_ac_alt': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'all_ac_ref': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'all_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'all_gtc': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'clinical_association': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'ea_ac_alt': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'ea_ac_ref': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'ea_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'ea_gtc': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'gts': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'read_depth': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'evs'", 'to': "orm['variants.Variant']"})
        },
        'variants.functionalclass': {
            'Meta': {'ordering': "['order']", 'object_name': 'FunctionalClass', 'db_table': "'variant_functional_class'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.polyphen2': {
            'Meta': {'object_name': 'PolyPhen2', 'db_table': "'polyphen2'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refaa': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'score': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'polyphen2'", 'to': "orm['variants.Variant']"})
        },
        'variants.sift': {
            'Meta': {'object_name': 'Sift', 'db_table': "'sift'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refaa': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'score': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'varaa': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'sift'", 'to': "orm['variants.Variant']"})
        },
        'variants.thousandg': {
            'Meta': {'object_name': 'ThousandG', 'db_table': "'1000g'"},
            'aa': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'ac': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'afr_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'amr_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'an': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'asn_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'eur_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'thousandg'", 'to': "orm['variants.Variant']"})
        },
        'variants.variant': {
            'Meta': {'unique_together': "(('chr', 'pos', 'ref', 'alt'),)", 'object_name': 'Variant', 'db_table': "'variant'"},
            'alt': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'db_table': "'variant_pubmed'", 'symmetrical': 'False'}),
            'chr': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genome.Chromosome']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'liftover': ('django.db.models.fields.NullBooleanField', [], {'null': 'True', 'blank': 'True'}),
            'md5': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'phenotypes': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['phenotypes.Phenotype']", 'through': "orm['variants.VariantPhenotype']", 'symmetrical': 'False'}),
            'pos': ('django.db.models.fields.IntegerField', [], {}),
            'ref': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'rsid': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.VariantType']", 'null': 'True'})
        },
        'variants.varianteffect': {
            'Meta': {'object_name': 'VariantEffect', 'db_table': "'variant_effect'"},
            'amino_acid_change': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'codon_change': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'effect': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.Effect']", 'null': 'True', 'blank': 'True'}),
            'exon': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Exon']", 'null': 'True', 'blank': 'True'}),
            'functional_class': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.FunctionalClass']", 'null': 'True', 'blank': 'True'}),
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']", 'null': 'True', 'blank': 'True'}),
            'hgvs_c': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'hgvs_p': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'transcript': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Transcript']", 'null': 'True', 'blank': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'effects'", 'null': 'True', 'to': "orm['variants.Variant']"})
        },
        'variants.variantphenotype': {
            'Meta': {'object_name': 'VariantPhenotype', 'db_table': "'variant_phenotype'"},
            'hgmd_id': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'phenotype': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['phenotypes.Phenotype']"}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'variant_phenotypes'", 'to': "orm['variants.Variant']"})
        },
        'variants.varianttype': {
            'Meta': {'ordering': "['order']", 'object_name': 'VariantType', 'db_table': "'variant_type'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        }
    }

    complete_apps = ['variants']


from django.db import models
from vdw.literature.models import PubMed


class Phenotype(models.Model):
    term = models.CharField(max_length=1000, unique=True)
    description = models.TextField(null=True, blank=True)
    hpo_id = models.IntegerField(null=True)

    articles = models.ManyToManyField(PubMed)

    class Meta(object):
        db_table = 'phenotype'


class PhenotypeThrough(models.Model):
    phenotype = models.ForeignKey(Phenotype)
    hgmd_id = models.CharField(max_length=30, null=True, blank=True,
                               db_index=True)

    class Meta(object):
        abstract = True



import os
import redis
import glob
import logging
from django_rq import get_worker
from rq import Worker
from redis.exceptions import ConnectionError
from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import DEFAULT_DB_ALIAS
from vdw.samples.pipeline.handlers import load_samples

SAMPLE_DIRS = getattr(settings, 'VARIFY_SAMPLE_DIRS', ())

log = logging.getLogger(__name__)


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--database', action='store', dest='database',
                    default=DEFAULT_DB_ALIAS,
                    help='Specifies the target database loading results.'),
        make_option('--max', action='store', dest='max', default=None,
                    type='int',
                    help='Specifies the maximum number of samples to queue.'),
        make_option('--startworkers', action='store_true', dest='startworkers',
                    default=False,
                    help='Include --startworkers flag to automatically start '
                         'the variant and default workers if they are not '
                         'already running. The workers will automatically '
                         'burst after the job is finished but will block the '
                         'terminal until the queues are emptied and all jobs '
                         'are finished.'),
    )

    def _queue(self, dirs, max_count, database, verbosity):
        count = 0
        skipped = 0
        scanned = 0

        # Walk the directory tree of each sample dirs to find all sample
        # directories with a valid MANIFEST file
        for source in dirs:
            log.debug('Scanning source directory: {0}'.format(source))

            for root, dirs, files in os.walk(source):
                if max_count and count == max_count:
                    return count, scanned

                if 'MANIFEST' not in files:
                    continue

                manifest_path = os.path.join(root, 'MANIFEST')

                try:
                    load_dict = load_samples(manifest_path, database)
                except Exception:
                    log.exception('error processing manifest {0}'
                                  .format(manifest_path))
                    continue

                if not load_dict:
                    continue

                count += load_dict['created']
                skipped += load_dict['skipped']
                if load_dict['created'] > 0:
                    if verbosity > 1:
                        log.debug('Queued sample: "{0}"'.format(root))
                elif verbosity > 2:
                    if load_dict['created'] == 0:
                        log.debug('Sample already loaded: "{0}"'.format(root))
                    else:
                        log.debug('Sample skipped: "{0}"'.format(root))

                scanned += 1

                # Print along the way since this is the only output for this
                # verbosity level
                if verbosity == 1:
                    log.debug(
                        "Queued {0} samples (max {1}) {2} skipped of {3} "
                        "scanned".format(count, max_count, skipped, scanned))

        return count, scanned

    def startWorkers(self):
        # Find the number of current workers
        queues = getattr(settings, 'RQ_QUEUES', {})
        default = queues['default'] if 'default' in queues else None
        variants = queues['variants'] if 'variants' in queues else None

        if not (queues and default and variants):
            log.warning('RQ_QUEUES settings could not be found')
            return

        # Create connections to redis to identify the workers
        def_connection = redis.Redis(host=default['HOST'],
                                     port=default['PORT'],
                                     db=default['DB'])
        var_connection = redis.Redis(host=variants['HOST'],
                                     port=variants['PORT'],
                                     db=variants['DB'])

        # Get all the workers connected with our redis server
        try:
            all_workers = Worker.all(def_connection) + \
                Worker.all(var_connection)
        except ConnectionError:
            log.warning('Could not connect to redis server to create workers. '
                        'Please make sure Redis server is running')
            return

        found_default = False
        found_variant = False

        # Loop through all the workers (even duplicates)
        for worker in all_workers:
            found_default = found_default or 'default' in worker.queue_names()
            found_variant = found_variant or 'variants' in worker.queue_names()

        # Start the required worker
        if not found_variant:
            log.debug('Did not find variants worker. Starting ... ')
            get_worker('variants').work(burst=True)

        if not found_default:
            log.debug('Did not find default worker. Starting ... ')
            get_worker('default').work(burst=True)

    def handle(self, *dirs, **options):
        database = options.get('database')
        max_count = options.get('max')
        verbosity = int(options.get('verbosity'))
        workers = options.get('startworkers')

        if not dirs:
            dirs = []
            for _dir in SAMPLE_DIRS:
                dirs.extend(glob.glob(_dir))

        count, scanned = self._queue(dirs, max_count, database, verbosity)

        if verbosity > 1:
            log.debug('Queued {0} samples (max {1}) of {2} scanned'.format(
                count, max_count, scanned))

        else:
            # Add a newline since the verbosity=1 output is written in-place
            # with a carriage return
            print ''

        # Start workers when appropriate
        if workers:
            self.startWorkers()



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Deleting model 'Genome'
        db.delete_table('genome')


    def backwards(self, orm):
        # Adding model 'Genome'
        db.create_table('genome', (
            ('released', self.gf('django.db.models.fields.DateField')(null=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('version', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=200)),
        ))
        db.send_create_signal('genome', ['Genome'])


    models = {
        'genome.chromosome': {
            'Meta': {'ordering': "['order']", 'object_name': 'Chromosome', 'db_table': "'chromosome'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '2', 'db_index': 'True'})
        },
        'genome.genotype': {
            'Meta': {'object_name': 'Genotype', 'db_table': "'genotype'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '3'})
        }
    }

    complete_apps = ['genome']


from vdw.management.commands import Subcommander


class Command(Subcommander):
    app_name = 'samples'

    subcommands = {
        'allele-freqs': 'allele_freqs',
        'gene-ranks': 'gene_ranks',
        'queue': 'queue',
        'delete-sample': 'delete_sample',
        'delete-project': 'delete_project',
    }



DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'vdwdb',
        'USER': 'vdw',
        'HOST': 'localhost',
    },
}

RQ_QUEUES = {
    'default': {
        'HOST': 'localhost',
        'PORT': 6379,
        'DB': 0,
    },
    'samples': {
        'HOST': 'localhost',
        'PORT': 6379,
        'DB': 0,
    },
    'variants': {
        'HOST': 'localhost',
        'PORT': 6379,
        'DB': 0,
    },
}



from vdw.conf import settings
from django.core.exceptions import ImproperlyConfigured

__version_info__ = {
    'major': 0,
    'minor': 4,
    'micro': 1,
    'releaselevel': 'final',
    'serial': 1
}


def get_version(short=False):
    assert __version_info__['releaselevel'] in ('alpha', 'beta', 'final')
    vers = ["%(major)i.%(minor)i.%(micro)i" % __version_info__, ]
    if __version_info__['releaselevel'] != 'final' and not short:
        vers.append('%s%i' % (__version_info__['releaselevel'][0],
                    __version_info__['serial']))
    return ''.join(vers)

__version__ = get_version()

if getattr(settings, 'VDW_GENOME_VERSION', None) is None:
    raise ImproperlyConfigured(
        'VDW_GENOME_VERSION must be defined in settings.')



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

class Migration(DataMigration):
    depends_on = (
        ('avocado', '0035_auto__add_field_datafield_indexable__add_field_dataconcept_indexable'),
    )

    def forwards(self, orm):
        "Write your forwards methods here."
        orm['avocado.DataField'].objects.get_or_create(
            app_name='variants',
            model_name='variantphenotype',
            field_name='hgmd_id',
            defaults={
                'name': 'HGMD',
                'published': True
            })

    def backwards(self, orm):
        "Write your backwards methods here."
        orm['avocado.DataField'].objects.filter(app_name='variants',
            model_name='variantphenotype', field_name='hgmd_id').delete()


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'avocado.datacategory': {
            'Meta': {'ordering': "('parent__order', 'parent__name', 'order', 'name')", 'object_name': 'DataCategory'},
            'archived': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_column': "'_order'", 'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['avocado.DataCategory']"}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'avocado.dataconcept': {
            'Meta': {'ordering': "('category__order', 'category__name', 'order', 'name')", 'object_name': 'DataConcept'},
            'archived': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['avocado.DataCategory']", 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'fields': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'concepts'", 'symmetrical': 'False', 'through': "orm['avocado.DataConceptField']", 'to': "orm['avocado.DataField']"}),
            'formatter_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'concepts+'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ident': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'indexable': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'internal': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'name_plural': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_column': "'_order'", 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'queryable': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'sites': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'concepts+'", 'blank': 'True', 'to': "orm['sites.Site']"}),
            'sortable': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'viewable': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        'avocado.dataconceptfield': {
            'Meta': {'ordering': "('order', 'name')", 'object_name': 'DataConceptField'},
            'concept': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'concept_fields'", 'to': "orm['avocado.DataConcept']"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'field': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'concept_fields'", 'to': "orm['avocado.DataField']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'name_plural': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_column': "'_order'", 'blank': 'True'})
        },
        'avocado.datacontext': {
            'Meta': {'object_name': 'DataContext'},
            'accessed': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 10, 16, 0, 0)'}),
            'count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'db_column': "'_count'"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'default': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'json': ('jsonfield.fields.JSONField', [], {'default': '{}', 'null': 'True', 'blank': 'True'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'forks'", 'null': 'True', 'to': "orm['avocado.DataContext']"}),
            'session': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'template': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'tree': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'datacontext+'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'avocado.datafield': {
            'Meta': {'ordering': "('category__order', 'category__name', 'order', 'name')", 'unique_together': "(('app_name', 'model_name', 'field_name'),)", 'object_name': 'DataField'},
            'app_name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'archived': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['avocado.DataCategory']", 'null': 'True', 'blank': 'True'}),
            'code_field_name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'data_version': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'enumerable': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'field_name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'fields+'", 'null': 'True', 'to': "orm['auth.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'indexable': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'internal': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'label_field_name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'model_name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'name_plural': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_column': "'_order'", 'blank': 'True'}),
            'order_field_name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'search_field_name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'sites': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'fields+'", 'blank': 'True', 'to': "orm['sites.Site']"}),
            'translator': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'unit': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'unit_plural': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'})
        },
        'avocado.dataquery': {
            'Meta': {'object_name': 'DataQuery'},
            'accessed': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'context_json': ('jsonfield.fields.JSONField', [], {'default': '{}', 'null': 'True', 'blank': 'True'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'default': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'distinct_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'forks'", 'null': 'True', 'to': "orm['avocado.DataQuery']"}),
            'public': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'record_count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'session': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'shared_users': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'shareddataquery+'", 'symmetrical': 'False', 'to': "orm['auth.User']"}),
            'template': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'tree': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'dataquery+'", 'null': 'True', 'to': "orm['auth.User']"}),
            'view_json': ('jsonfield.fields.JSONField', [], {'default': '{}', 'null': 'True', 'blank': 'True'})
        },
        'avocado.dataview': {
            'Meta': {'object_name': 'DataView'},
            'accessed': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2014, 10, 16, 0, 0)'}),
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'default': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'json': ('jsonfield.fields.JSONField', [], {'default': '{}', 'null': 'True', 'blank': 'True'}),
            'keywords': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'forks'", 'null': 'True', 'to': "orm['avocado.DataView']"}),
            'session': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'template': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'dataview+'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'avocado.revision': {
            'Meta': {'ordering': "('-timestamp',)", 'object_name': 'Revision'},
            'changes': ('jsonfield.fields.JSONField', [], {'null': 'True', 'blank': 'True'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'data': ('jsonfield.fields.JSONField', [], {'null': 'True', 'blank': 'True'}),
            'deleted': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'session_key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'null': 'True', 'blank': 'True'}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'+revision'", 'null': 'True', 'to': "orm['auth.User']"})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'genes.exon': {
            'Meta': {'object_name': 'Exon', 'db_table': "'exon'"},
            'end': ('django.db.models.fields.IntegerField', [], {}),
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'index': ('django.db.models.fields.IntegerField', [], {}),
            'start': ('django.db.models.fields.IntegerField', [], {})
        },
        'genes.gene': {
            'Meta': {'object_name': 'Gene', 'db_table': "'gene'"},
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'db_table': "'gene_pubmed'", 'symmetrical': 'False'}),
            'chr': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genome.Chromosome']"}),
            'families': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.GeneFamily']", 'symmetrical': 'False', 'blank': 'True'}),
            'hgnc_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'phenotypes': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['phenotypes.Phenotype']", 'through': "orm['genes.GenePhenotype']", 'symmetrical': 'False'}),
            'symbol': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            'synonyms': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.Synonym']", 'db_table': "'gene_synonym'", 'symmetrical': 'False'})
        },
        'genes.genefamily': {
            'Meta': {'object_name': 'GeneFamily', 'db_table': "'gene_family'"},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'tag': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True'})
        },
        'genes.genephenotype': {
            'Meta': {'object_name': 'GenePhenotype', 'db_table': "'gene_phenotype'"},
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']"}),
            'hgmd_id': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'phenotype': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['phenotypes.Phenotype']"})
        },
        'genes.synonym': {
            'Meta': {'object_name': 'Synonym', 'db_table': "'synonym'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'})
        },
        'genes.transcript': {
            'Meta': {'object_name': 'Transcript', 'db_table': "'transcript'"},
            'coding_end': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'coding_end_status': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'coding_start': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'coding_start_status': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True', 'blank': 'True'}),
            'end': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'exon_count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'exons': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['genes.Exon']", 'db_table': "'transcript_exon'", 'symmetrical': 'False'}),
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refseq_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'start': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'strand': ('django.db.models.fields.CharField', [], {'max_length': '1', 'null': 'True', 'blank': 'True'})
        },
        'genome.chromosome': {
            'Meta': {'ordering': "['order']", 'object_name': 'Chromosome', 'db_table': "'chromosome'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '2', 'db_index': 'True'})
        },
        'literature.pubmed': {
            'Meta': {'object_name': 'PubMed', 'db_table': "'pubmed'"},
            'pmid': ('django.db.models.fields.IntegerField', [], {'primary_key': 'True'})
        },
        'phenotypes.phenotype': {
            'Meta': {'object_name': 'Phenotype', 'db_table': "'phenotype'"},
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'symmetrical': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'hpo_id': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'term': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '1000'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'variants.effect': {
            'Meta': {'ordering': "['order']", 'object_name': 'Effect', 'db_table': "'effect'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'impact': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.EffectImpact']", 'null': 'True', 'blank': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'region': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.EffectRegion']", 'null': 'True', 'blank': 'True'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.effectimpact': {
            'Meta': {'ordering': "['order']", 'object_name': 'EffectImpact', 'db_table': "'effect_impact'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.effectregion': {
            'Meta': {'ordering': "['order']", 'object_name': 'EffectRegion', 'db_table': "'effect_region'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.evs': {
            'Meta': {'object_name': 'EVS', 'db_table': "'evs'"},
            'aa_ac_alt': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'aa_ac_ref': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'aa_gtc': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'aa_maf': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'all_ac_alt': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'all_ac_ref': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'all_gtc': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'all_maf': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'clinical_association': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'ea_ac_alt': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'ea_ac_ref': ('django.db.models.fields.CharField', [], {'max_length': '20', 'null': 'True'}),
            'ea_gtc': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'ea_maf': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'gts': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'read_depth': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'evs'", 'to': "orm['variants.Variant']"})
        },
        'variants.functionalclass': {
            'Meta': {'ordering': "['order']", 'object_name': 'FunctionalClass', 'db_table': "'variant_functional_class'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'variants.polyphen2': {
            'Meta': {'object_name': 'PolyPhen2', 'db_table': "'polyphen2'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refaa': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'score': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'polyphen2'", 'to': "orm['variants.Variant']"})
        },
        'variants.sift': {
            'Meta': {'object_name': 'Sift', 'db_table': "'sift'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'refaa': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'score': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'varaa': ('django.db.models.fields.CharField', [], {'max_length': '2', 'null': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'sift'", 'to': "orm['variants.Variant']"})
        },
        'variants.thousandg': {
            'Meta': {'object_name': 'ThousandG', 'db_table': "'1000g'"},
            'aa': ('django.db.models.fields.TextField', [], {'null': 'True'}),
            'ac': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'afr_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'amr_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'an': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'asn_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'eur_af': ('django.db.models.fields.FloatField', [], {'null': 'True', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'thousandg'", 'to': "orm['variants.Variant']"})
        },
        'variants.variant': {
            'Meta': {'unique_together': "(('chr', 'pos', 'ref', 'alt'),)", 'object_name': 'Variant', 'db_table': "'variant'"},
            'alt': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'db_table': "'variant_pubmed'", 'symmetrical': 'False'}),
            'chr': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genome.Chromosome']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'liftover': ('django.db.models.fields.NullBooleanField', [], {'null': 'True', 'blank': 'True'}),
            'md5': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'phenotypes': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['phenotypes.Phenotype']", 'through': "orm['variants.VariantPhenotype']", 'symmetrical': 'False'}),
            'pos': ('django.db.models.fields.IntegerField', [], {}),
            'ref': ('django.db.models.fields.TextField', [], {'db_index': 'True'}),
            'rsid': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.VariantType']", 'null': 'True'})
        },
        'variants.varianteffect': {
            'Meta': {'object_name': 'VariantEffect', 'db_table': "'variant_effect'"},
            'amino_acid_change': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'codon_change': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'effect': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.Effect']", 'null': 'True', 'blank': 'True'}),
            'exon': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Exon']", 'null': 'True', 'blank': 'True'}),
            'functional_class': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['variants.FunctionalClass']", 'null': 'True', 'blank': 'True'}),
            'gene': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Gene']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'transcript': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['genes.Transcript']", 'null': 'True', 'blank': 'True'}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'effects'", 'null': 'True', 'to': "orm['variants.Variant']"})
        },
        'variants.variantphenotype': {
            'Meta': {'object_name': 'VariantPhenotype', 'db_table': "'variant_phenotype'"},
            'hgmd_id': ('django.db.models.fields.CharField', [], {'max_length': '30', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'phenotype': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['phenotypes.Phenotype']"}),
            'variant': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'variant_phenotypes'", 'to': "orm['variants.Variant']"})
        },
        'variants.varianttype': {
            'Meta': {'ordering': "['order']", 'object_name': 'VariantType', 'db_table': "'variant_type'"},
            'code': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        }
    }

    complete_apps = ['variants']



import logging
import tempfile
try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict
from optparse import make_option
from django.db import transaction, connections, DEFAULT_DB_ALIAS
from django.core.management.base import BaseCommand
from vdw.variants.models import EVS, Sift, ThousandG, PolyPhen2

log = logging.getLogger(__name__)


def _write_tmp(cursor, row_handler=lambda x: x):
    tmp = tempfile.NamedTemporaryFile()
    count = 0

    while True:
        rows = cursor.fetchmany(100)
        if not rows:
            break

        for row in rows:
            row = row_handler(row)
            cleaned = [str(x) if x is not None else '\N' for x in row]
            tmp.write('\t'.join(cleaned) + '\n')
            count += 1

        tmp.flush()
    tmp.seek(0)

    return tmp, count


class Command(BaseCommand):
    """
    Cleans and loads genes from various sources into a uniform set of tables.
    """
    option_list = BaseCommand.option_list + (
        make_option('--database', action='store', dest='database',
                    default=DEFAULT_DB_ALIAS,
                    help='Nominates a database to print the SQL for. Defaults '
                         'to the "default" database.'),
        make_option('--sift', action='store_true', default=False,
                    help='Reload SIFT annotations'),
        make_option('--polyphen2', action='store_true', default=False,
                    help='Reload PolyPhen2 annotations'),
        make_option('--1000g', action='store_true', default=False,
                    help='Reload 1000g annotations'),
        make_option('--evs', action='store_true', default=False,
                    help='Reload EVS annotations'),
        make_option('--truncate', action='store_true', default=False,
                    help='Truncate tables before reloading'),
    )

    def load_1000g(self, cursor, truncate=False):
        db_table = ThousandG._meta.db_table

        if truncate:
            cursor.execute('TRUNCATE "{0}"'.format(db_table))
            log.debug('Truncated table "{0}"'.format(db_table))

        cursor.execute('''
            SELECT DISTINCT ON (variant.id) variant.id, r.an, r.ac, r.af,
                r.aa, r.amr_af, r.asn_af, r.afr_af, r.eur_af
            FROM "variant"
                INNER JOIN "raw"."1000g" r ON ("variant".md5 = r."md5")
                LEFT OUTER JOIN "1000g" ON
                    ("1000g"."variant_id" = "variant"."id")
            WHERE "1000g"."id" IS NULL
            ORDER BY variant.id
        ''')

        tmp, count = _write_tmp(cursor)
        cursor.copy_from(
            tmp, '"1000g"', columns=['variant_id', 'an', 'ac', 'af', 'aa',
                                     'amr_af', 'asn_af', 'afr_af', 'eur_af'])
        tmp.close()
        return count

    load_1000g.short_name = '1000g'

    def load_evs(self, cursor, truncate=False):
        db_table = EVS._meta.db_table

        if truncate:
            cursor.execute('TRUNCATE "{0}"'.format(db_table))
            log.debug('Truncated table "{0}"'.format(db_table))

        # MAFs are divided by 100 since they are percentages to begin with
        cursor.execute('''
            SELECT DISTINCT ON (variant.id) variant.id, r.ea_ac_ref,
                r.ea_ac_alt, r.aa_ac_ref, r.aa_ac_alt, r.all_ac_ref,
                r.all_ac_alt, r.ea_maf / 100.0, r.aa_maf / 100.0,
                r.all_maf / 100.0, r.gts, r.ea_gtc, r.aa_gtc, r.all_gtc,
                r.clinical_association
            FROM "variant"
                INNER JOIN "raw"."evs" r ON ("variant".md5 = r."md5")
                LEFT OUTER JOIN "evs" ON ("evs"."variant_id" = "variant"."id")
            WHERE "evs"."id" IS NULL
            ORDER BY variant.id
        ''')

        # Note, *_af are actually minor allele frequencies by default.
        # The handler checks to see if the reference count is less than
        # the alternate and sets the AF if the reference count is less than
        # the alternate
        columns = ['variant_id', 'ea_ac_ref', 'ea_ac_alt', 'aa_ac_ref',
                   'aa_ac_alt', 'all_ac_ref', 'all_ac_alt', 'ea_af', 'aa_af',
                   'all_af', 'gts', 'ea_gtc', 'aa_gtc', 'all_gtc',
                   'clinical_association']

        def compare_counts(ref, alt):
            "Compares allele counts and handles heterozygotes."
            ref = int(ref)
            for _alt in alt.split(','):
                if ref < int(_alt):
                    return True
            return False

        def row_handler(row):
            record = OrderedDict(zip(columns, row))

            # All
            if compare_counts(record['all_ac_ref'], record['all_ac_alt']):
                record['all_af'] = 1 - record['all_af']

            # European
            if compare_counts(record['ea_ac_ref'], record['ea_ac_alt']):
                record['ea_af'] = 1 - record['ea_af']

            # African American
            if compare_counts(record['aa_ac_ref'], record['aa_ac_alt']):
                record['aa_af'] = 1 - record['aa_af']

            return record.values()

        tmp, count = _write_tmp(cursor, row_handler)
        cursor.copy_from(tmp, 'evs', columns=columns)
        tmp.close()
        return count

    load_evs.short_name = 'EVS'

    def load_polyphen2(self, cursor, truncate=False):
        db_table = PolyPhen2._meta.db_table

        if truncate:
            cursor.execute('TRUNCATE "{0}"'.format(db_table))
            log.debug('Truncated table "{0}"'.format(db_table))

        cursor.execute('''
            SELECT variant.id, r.score, r.refaa
            FROM "variant"
                INNER JOIN "raw"."polyphen2" r ON ("variant".md5 = r."md5")
                LEFT OUTER JOIN "polyphen2" ON
                    ("polyphen2"."variant_id" = "variant"."id")
            WHERE "polyphen2"."id" IS NULL
        ''')

        tmp, count = _write_tmp(cursor)
        cursor.copy_from(tmp, 'polyphen2',
                         columns=['variant_id', 'score', 'refaa'])
        tmp.close()
        return count

    load_polyphen2.short_name = 'PolyPhen2'

    def load_sift(self, cursor, truncate=False):
        db_table = Sift._meta.db_table

        if truncate:
            cursor.execute('TRUNCATE "{0}"'.format(db_table))
            log.debug('Truncated table "{0}"'.format(db_table))

        cursor.execute('''
            SELECT variant.id, r.score, r.refaa, r.varaa
            FROM "variant"
                INNER JOIN "raw"."sift" r ON ("variant".md5 = r."md5")
                LEFT OUTER JOIN "sift" ON
                    ("sift"."variant_id" = "variant"."id")
            WHERE "sift"."id" IS NULL
        ''')

        tmp, count = _write_tmp(cursor)
        cursor.copy_from(tmp, 'sift',
                         columns=['variant_id', 'score', 'refaa', 'varaa'])
        tmp.close()
        return count

    load_sift.short_name = 'SIFT'

    def handle(self, *args, **options):
        using = options.get('database')

        load_1000g = self.load_1000g if options.get('1000g') else None
        load_polyphen2 = \
            self.load_polyphen2 if options.get('polyphen2') else None
        load_evs = self.load_evs if options.get('evs') else None
        load_sift = self.load_sift if options.get('sift') else None
        truncate = options.get('truncate')

        cursor = connections[using].cursor()

        for handler in (load_1000g, load_polyphen2, load_evs, load_sift):
            if not handler:
                continue

            with transaction.commit_manually(using):
                try:
                    log.debug('Loading {0}...'.format(handler.short_name))
                    count = handler(cursor, truncate)
                    log.debug('{0} rows copied'.format(count))
                    transaction.commit()
                except Exception, e:
                    transaction.rollback()
                    log.exception(e.message)



from django.db import models
from django.contrib.auth.models import User
from objectset.models import ObjectSet, SetObject
from vdw.literature.models import PubMed
from vdw.genome.models import Chromosome
from vdw.phenotypes.models import Phenotype, PhenotypeThrough
from .managers import GeneManager


class GeneFamily(models.Model):
    "Gene family tags and descriptions."
    tag = models.CharField(max_length=30, null=True)
    description = models.CharField(max_length=200, null=True)

    class Meta(object):
        db_table = 'gene_family'


class Synonym(models.Model):
    """Model which contains known alternate gene names and symbols for
    the canonical genes. This can be used as an index for search-related
    queries.
    """
    # Call it a label since this may be a symbol, a name or something else
    label = models.CharField(max_length=255, db_index=True)

    class Meta(object):
        db_table = 'synonym'


class Gene(models.Model):
    """Unified gene model. This includes data from multiple sources with
    the appropriate `id` defined to which references the source. If multiple
    sources contain have overlap, the respective `id`s will be filled in.

    The canonical source is HGNC, which approves gene names and symbols, the
    `approved` flag should be set if this is the approved gene name and
    symbol by HGNC.
    """
    chr = models.ForeignKey(Chromosome)
    symbol = models.CharField(max_length=255, db_index=True)
    name = models.TextField('full name', blank=True)
    hgnc_id = models.IntegerField('HGNC ID', null=True, blank=True)

    # Via the HGNC documentation: "Families/groups may be either structural or
    # functional, therefore a gene may belong to more than one family/group"
    families = models.ManyToManyField(GeneFamily, blank=True)

    # Literature
    articles = models.ManyToManyField(PubMed, db_table='gene_pubmed')

    # Synonyms
    synonyms = models.ManyToManyField(Synonym, db_table='gene_synonym')

    # Phenotypes
    phenotypes = models.ManyToManyField(Phenotype, through='GenePhenotype')

    objects = GeneManager()

    class Meta(object):
        db_table = 'gene'

    def __unicode__(self):
        return self.symbol

    def approved(self):
        return self.hgnc_id is not None

    def hgnc_url(self):
        if self.hgnc_id:
            return 'http://www.genenames.org/data/hgnc_data.php?hgnc_id=' + \
                str(self.hgnc_id)


class GenePhenotype(PhenotypeThrough):
    gene = models.ForeignKey(Gene)

    class Meta(object):
        db_table = 'gene_phenotype'


class Exon(models.Model):
    "Gene-specific exon region"
    gene = models.ForeignKey(Gene)
    index = models.IntegerField('exon index')
    start = models.IntegerField('exon start position')
    end = models.IntegerField('exon end position')

    class Meta(object):
        db_table = 'exon'


class Transcript(models.Model):
    "Gene transcripts"
    refseq_id = models.CharField(max_length=100, unique=True)
    strand = models.CharField(max_length=1, null=True, blank=True,
                              help_text='+ or - for strand')
    start = models.IntegerField('transcript start position', null=True,
                                blank=True)
    end = models.IntegerField('transcript end position', null=True, blank=True)
    coding_start = models.IntegerField('coding region start position',
                                       null=True, blank=True)
    coding_end = models.IntegerField('coding region end position', null=True,
                                     blank=True)
    coding_start_status = models.CharField('coding region start status',
                                           max_length=20, null=True,
                                           blank=True)
    coding_end_status = models.CharField('coding region end status',
                                         max_length=20, null=True, blank=True)
    exon_count = models.IntegerField('number of exons', null=True, blank=True)

    gene = models.ForeignKey(Gene, null=True, blank=True)
    exons = models.ManyToManyField(Exon, db_table='transcript_exon')

    class Meta(object):
        db_table = 'transcript'

    def ncbi_url(self):
        return 'http://www.ncbi.nlm.nih.gov/nuccore/' + self.refseq_id


class GeneSet(ObjectSet):
    user = models.ForeignKey(User, null=True, blank=True)
    name = models.CharField(max_length=100, null=True, blank=True)
    genes = models.ManyToManyField(Gene, through='GeneSetObject')

    published = models.BooleanField(default=True)

    set_object_rel = 'genes'
    label_field = 'name'

    def __unicode__(self):
        return unicode(self.name)

    class Meta(object):
        db_table = 'geneset'
        ordering = ('user', 'name',)


class GeneSetObject(SetObject):
    object_set = models.ForeignKey(GeneSet, db_column='set_id')
    set_object = models.ForeignKey(Gene, db_column='object_id')

    class Meta(object):
        db_table = 'geneset_setobject'



import sys
from optparse import NO_DEFAULT, OptionParser
from django.core.management.base import CommandError, BaseCommand, \
    handle_default_options
from django.utils.importlib import import_module


class Subcommander(BaseCommand):
    help = "A wrapper for subscommands"

    subcommands = {}

    import_template = 'vdw.{app}.management.subcommands.{module}'

    def print_subcommands(self, prog_name):
        usage = ['', 'Available subcommands:']
        for name in sorted(self.subcommands):
            usage.append('  {0}'.format(name))
        return '\n'.join(usage)

    def usage(self, subcommand):
        usage = '%prog {0} subcommand [options] [args]'.format(subcommand)
        if self.help:
            return '{0}\n\n{1}'.format(usage, self.help)
        return usage

    def print_help(self, prog_name, subcommand):
        super(Subcommander, self).print_help(prog_name, subcommand)
        sys.stdout.write('{0}\n\n'.format(self.print_subcommands(prog_name)))

    def get_subcommand(self, subcommand):
        try:
            module = import_module(self.import_template.format(
                app=self.app_name, module=self.subcommands[subcommand]))
            return module.Command()
        except KeyError:
            raise CommandError('Unknown subcommand: {0} {1}'
                               .format(self.app_name, subcommand))

    def run_from_argv(self, argv):
        """
        Set up any environment changes requested (e.g., Python path and Django
        settings), then run this command.
        """
        if (len(argv) > 2 and not argv[2].startswith('-') and
                argv[2] in self.subcommands.keys()):
            subcommand = argv[2]
            klass = self.get_subcommand(subcommand)
            parser = OptionParser(
                prog=argv[0],
                usage=klass.usage('{0} {1}'.format(argv[1], subcommand)),
                version=klass.get_version(), option_list=klass.option_list)
            options, args = parser.parse_args(argv[3:])
            args = [subcommand] + args
        else:
            parser = self.create_parser(argv[0], argv[1])
            options, args = parser.parse_args(argv[2:])

        handle_default_options(options)
        self.execute(*args, **options.__dict__)

    def handle(self, *args, **options):
        if not args or args[0] not in self.subcommands.keys():
            return self.print_help('./manage.py', self.app_name)
        subcommand, args = args[0], args[1:]

        klass = self.get_subcommand(subcommand)
        # Grab out a list of defaults from the options. optparse does this for
        # us when the script runs from the command line, but since
        # call_command can be called programatically, we need to simulate the
        # loading and handling of defaults (see #10080 for details).
        defaults = {}
        for opt in klass.option_list:
            if opt.default is NO_DEFAULT:
                defaults[opt.dest] = None
            else:
                defaults[opt.dest] = opt.default
        defaults.update(options)

        return klass.execute(*args, **defaults)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    depends_on = (
        ('literature', '0001_initial'),
    )

    def forwards(self, orm):

        # Adding model 'Phenotype'
        db.create_table('phenotype', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('term', self.gf('django.db.models.fields.CharField')(unique=True, max_length=1000)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('hpo_id', self.gf('django.db.models.fields.IntegerField')(null=True)),
        ))
        db.send_create_signal('phenotypes', ['Phenotype'])

        # Adding M2M table for field articles on 'Phenotype'
        db.create_table('phenotype_articles', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('phenotype', models.ForeignKey(orm['phenotypes.phenotype'], null=False)),
            ('pubmed', models.ForeignKey(orm['literature.pubmed'], null=False))
        ))
        db.create_unique('phenotype_articles', ['phenotype_id', 'pubmed_id'])


    def backwards(self, orm):

        # Deleting model 'Phenotype'
        db.delete_table('phenotype')

        # Removing M2M table for field articles on 'Phenotype'
        db.delete_table('phenotype_articles')


    models = {
        'literature.pubmed': {
            'Meta': {'object_name': 'PubMed', 'db_table': "'pubmed'"},
            'pmid': ('django.db.models.fields.IntegerField', [], {'primary_key': 'True'})
        },
        'phenotypes.phenotype': {
            'Meta': {'object_name': 'Phenotype', 'db_table': "'phenotype'"},
            'articles': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['literature.PubMed']", 'symmetrical': 'False'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'hpo_id': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'term': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '1000'})
        }
    }

    complete_apps = ['phenotypes']



from django.conf import settings
from avocado.core.loader import autodiscover
from .utils import Channel, job, ManifestReader     # noqa

PIPELINE_CACHE_ALIAS = \
    getattr(settings, 'VARIFY_PIPELINE_CACHE_ALIAS', 'vdw.pipeline')

# Creates a pipeline component registry. `pipeline` modules or packages in
# apps that are _installed_ will be autoloaded which enables auto-registering
# components to the pipeline.
autodiscover('pipeline')



from global_settings import *   # noqa

try:
    from local_settings import *    # noqa
except ImportError:
    import warnings
    warnings.warn(
        'Local settings have not been found (vdw.conf.local_settings)')
